import tkinter as tk
import webbrowser

from subsearch.data import __data__, __version__
from subsearch.gui import base_root, tk_data, tk_tools
from subsearch.utils import current_user, raw_config, raw_registry, updates

TKWINDOW = tk_data.Window()
TKCOLOR = tk_data.Color()
TKFONT = tk_data.Font()
TKMISC = tk_data.Misc()

LANGUAGES = raw_config.get_config_key("languages")
CURRENT_LANGUAGE = raw_config.get_config_key("current_language")
HEARING_IMPARED = raw_config.get_config_key("hearing_impaired")
PCT = raw_config.get_config_key("percentage")
SHOW_TERMINAL = raw_config.get_config_key("show_terminal")
CONTEXT_MENU_ICON = raw_config.get_config_key("context_menu_icon")
DL_WINDOW = raw_config.get_config_key("show_download_window")
AVAIL_EXT = raw_config.get_config_key("file_ext")
PROVIDERS = raw_config.get_config_key("providers")


class SelectLanguage(tk.Frame):
    def __init__(self, parent):
        tk.Frame.__init__(self, parent)
        self.configure(bg=TKCOLOR.dark_grey)
        self.string_var = tk.StringVar()
        self.string_var.set(f"{CURRENT_LANGUAGE}")
        number_of_buttons = len(LANGUAGES)
        self.rownum = 0
        self.colnum = 0
        self.btn_active = None
        for i, key in zip(range(number_of_buttons), LANGUAGES.keys()):
            self.rownum += 1
            if self.rownum == 15:
                self.rownum = 1
                self.colnum += 1

            btn = tk.Button(
                self,
                font=TKFONT.cas8b,
                text=key,
                bd=0,
                bg=TKCOLOR.light_black,
                fg=TKCOLOR.white_grey,
                activebackground=TKCOLOR.orange,
                height=2,
                width=18,
            )
            btn.grid(row=self.rownum + 1, column=self.colnum, pady=2)
            if btn["text"] == self.string_var.get():
                btn.configure(fg=TKCOLOR.yellow)
                self.btn_active = btn

            # btn.bind("<Button-1>", self.enter_button)
            btn.bind("<Enter>", self.enter_button)
            btn.bind("<Leave>", self.leave_button)
            tk_tools.row_col_minsize(self, 18)

    def enter_button(self, event):
        btn = event.widget
        btn.configure(bg=TKCOLOR.black, fg=TKCOLOR.orange)
        btn.bind("<ButtonRelease>", self.set_current_language)

    def leave_button(self, event):
        btn = event.widget
        btn.configure(bg=TKCOLOR.light_black, fg=TKCOLOR.white_grey)
        if btn["text"] == self.string_var.get():
            btn.configure(fg=TKCOLOR.yellow)

    # set language
    def set_current_language(self, event):
        btn = event.widget
        event.widget.configure(fg=TKCOLOR.yellow)
        self.string_var.set(btn.cget("text"))
        update_svar = btn["text"]
        raw_config.set_config_key_value("current_language", update_svar)
        if btn["text"] == self.string_var.get():
            btn.configure(fg=TKCOLOR.yellow)
            self.btn_active.configure(fg=TKCOLOR.white_grey)
            self.btn_active = btn


class HearingImparedSubs(tk.Frame):
    def __init__(self, parent):
        tk.Frame.__init__(self, parent)
        self.configure(bg=TKCOLOR.dark_grey)
        self.string_var = tk.StringVar()
        self.string_var.set(f"{HEARING_IMPARED}")
        label = tk.Label(self, text="Hearing impaired")
        label.configure(bg=TKCOLOR.dark_grey, fg=TKCOLOR.white_grey, font=TKFONT.cas8b)
        label.grid(row=0, column=0, sticky="w", padx=2, pady=2)
        self.clabel = tk.Label(self, textvariable=self.string_var)
        self.clabel.configure(bg=TKCOLOR.dark_grey, font=TKFONT.cas8b)
        self.clabel.grid(row=0, column=1, sticky="nsew", padx=2, pady=2)
        tk_tools.ColorPicker(self.string_var, self.clabel)
        btn_true = tk.Button(
            self,
            font=TKFONT.cas8b,
            text="True",
            bd=0,
            bg=TKCOLOR.light_black,
            fg=TKCOLOR.white_grey,
            activebackground=TKCOLOR.green,
            height=2,
            width=18,
        )
        btn_true.grid(row=0, column=4, pady=2)
        btn_both = tk.Button(
            self,
            font=TKFONT.cas8b,
            text="Both",
            bd=0,
            bg=TKCOLOR.light_black,
            fg=TKCOLOR.white_grey,
            activebackground=TKCOLOR.blue,
            height=2,
            width=18,
        )
        btn_both.grid(row=0, column=3, pady=2)
        btn_false = tk.Button(
            self,
            font=TKFONT.cas8b,
            text="False",
            bd=0,
            bg=TKCOLOR.light_black,
            fg=TKCOLOR.white_grey,
            activebackground=TKCOLOR.red,
            height=2,
            width=18,
        )
        btn_false.grid(row=0, column=2, pady=2)
        btn_true.bind("<Enter>", self.enter_button)
        btn_true.bind("<Leave>", self.leave_button)
        btn_both.bind("<Enter>", self.enter_button)
        btn_both.bind("<Leave>", self.leave_button)
        btn_false.bind("<Enter>", self.enter_button)
        btn_false.bind("<Leave>", self.leave_button)
        tk_tools.row_col_minsize(self)

    def enter_button(self, event):
        btn = event.widget
        if btn["text"] == "True":
            btn.configure(bg=TKCOLOR.black, fg=TKCOLOR.green)
            btn.bind("<ButtonRelease>", self.button_set_true)
        if btn["text"] == "Both":
            btn.configure(bg=TKCOLOR.black, fg=TKCOLOR.blue)
            btn.bind("<ButtonRelease>", self.button_set_both)
        if btn["text"] == "False":
            btn.configure(bg=TKCOLOR.black, fg=TKCOLOR.red)
            btn.bind("<ButtonRelease>", self.button_set_false)

    def leave_button(self, event):
        btn = event.widget
        btn.configure(bg=TKCOLOR.light_black, fg=TKCOLOR.white_grey)

    def button_set_true(self, event):
        self.string_var.set(f"True")
        tk_tools.ColorPicker(self.string_var, self.clabel)
        raw_config.set_config_key_value("hearing_impaired", True)

    def button_set_false(self, event):
        self.string_var.set(f"False")
        tk_tools.ColorPicker(self.string_var, self.clabel)
        raw_config.set_config_key_value("hearing_impaired", False)

    def button_set_both(self, event):
        self.string_var.set(f"Both")
        tk_tools.ColorPicker(self.string_var, self.clabel)
        update_svar = self.string_var.get().split(" ")[0]
        raw_config.set_config_key_value("hearing_impaired", update_svar)


class SearchThreshold(tk.Frame):
    def __init__(self, parent):
        tk.Frame.__init__(self, parent)
        self.configure(bg=TKCOLOR.dark_grey)
        self.string_var = tk.StringVar()
        self.string_var.set(f"{PCT} %")
        self.pct = PCT
        label = tk.Label(self, text="Search threshold")
        label.configure(bg=TKCOLOR.dark_grey, fg=TKCOLOR.white_grey, font=TKFONT.cas8b)
        label.grid(row=0, column=0, sticky="w", padx=2, pady=2)
        self.clabel = tk.Label(self, textvariable=self.string_var)
        self.clabel.configure(bg=TKCOLOR.dark_grey, font=TKFONT.cas8b)
        self.clabel.grid(row=0, column=1, sticky="nsew", padx=2, pady=2)
        tk_tools.ColorPicker(self.string_var, self.clabel, self.pct)
        btn_add = tk.Button(
            self,
            font=TKFONT.cas8b,
            text="+",
            bd=0,
            bg=TKCOLOR.light_black,
            fg=TKCOLOR.white_grey,
            activebackground=TKCOLOR.green,
            height=2,
            width=18,
        )
        btn_add.grid(row=0, column=4, pady=2)
        btn_sub = tk.Button(
            self,
            font=TKFONT.cas8b,
            text="-",
            bd=0,
            bg=TKCOLOR.light_black,
            fg=TKCOLOR.white_grey,
            activebackground=TKCOLOR.red,
            height=2,
            width=18,
        )
        btn_sub.grid(row=0, column=2, pady=2)
        btn_add.bind("<Enter>", self.enter_button)
        btn_add.bind("<Leave>", self.leave_button)
        btn_sub.bind("<Enter>", self.enter_button)
        btn_sub.bind("<Leave>", self.leave_button)
        tk_tools.row_col_minsize(self)

    def enter_button(self, event):
        btn = event.widget
        if btn["text"] == "+":
            btn.configure(bg=TKCOLOR.black, fg=TKCOLOR.green)
            btn.bind("<ButtonRelease>", self.button_add_5)
        if btn["text"] == "-":
            btn.configure(bg=TKCOLOR.black, fg=TKCOLOR.red)
            btn.bind("<ButtonRelease>", self.button_sub_5)

    def leave_button(self, event):
        btn = event.widget
        btn.configure(bg=TKCOLOR.light_black, fg=TKCOLOR.white_grey)
        tk_tools.ColorPicker(self.string_var, self.clabel)

    def button_add_5(self, event):
        self.pct += 5 if self.pct < 100 else 0
        self.string_var.set(f"{self.pct} %")

        tk_tools.ColorPicker(self.string_var, self.clabel, self.pct)
        update_svar = int(self.pct)
        raw_config.set_config_key_value("percentage", update_svar)

    def button_sub_5(self, event):
        self.pct -= 5 if self.pct > 0 else 0
        self.string_var.set(f"{self.pct} %")
        tk_tools.ColorPicker(self.string_var, self.clabel, self.pct)
        update_svar = int(self.pct)
        raw_config.set_config_key_value("percentage", update_svar)


class Providers(tk.Frame):
    def __init__(self, parent):
        tk.Frame.__init__(self, parent)
        self.configure(bg=TKCOLOR.dark_grey)
        self.data = raw_config.get_config()
        number_of_buttons = len(PROVIDERS.items())
        label = tk.Label(self, text="Search providers")
        label.configure(bg=TKCOLOR.dark_grey, fg=TKCOLOR.white_grey, font=TKFONT.cas8b)
        label.grid(row=1, column=0, sticky="w", padx=2, pady=2)
        self.rownum = 0
        self.colnum = 2
        for i, key in zip(range(number_of_buttons), PROVIDERS.keys()):
            self.rownum += 1
            if self.rownum > 1:
                self.rownum = 1
                self.colnum += 1

            btn = tk.Button(
                self,
                font=TKFONT.cas8b,
                text=key,
                bd=0,
                bg=TKCOLOR.light_black,
                fg=TKCOLOR.white_grey,
                activebackground=TKCOLOR.orange,
                height=2,
                width=18,
            )
            btn.grid(row=self.rownum, column=self.colnum, pady=2)
            if self.data["providers"][key] is False:
                btn.configure(fg=TKCOLOR.red)
            elif self.data["providers"][key] is True:
                btn.configure(fg=TKCOLOR.green)

            # btn.bind("<Button-1>", self.enter_button)
            btn.bind("<Enter>", self.enter_button)
            btn.bind("<Leave>", self.leave_button)
            tk_tools.row_col_minsize(self)

    def enter_button(self, event):
        btn = event.widget
        btn.configure(bg=TKCOLOR.black, fg=TKCOLOR.orange)
        btn.bind("<ButtonPress>", self.press_button)

    def leave_button(self, event):
        btn = event.widget
        key = btn["text"]
        if self.data["providers"][key] is True:
            btn.configure(bg=TKCOLOR.light_black, fg=TKCOLOR.green)
        if self.data["providers"][key] is False:
            btn.configure(bg=TKCOLOR.light_black, fg=TKCOLOR.red)

    def press_button(self, event):
        btn = event.widget
        key = btn["text"]
        if self.data["providers"][key] is True:
            btn.configure(bg=TKCOLOR.light_black, fg=TKCOLOR.green)
            btn.bind("<ButtonRelease>", self.toggle_providers)
        if self.data["providers"][key] is False:
            btn.configure(bg=TKCOLOR.light_black, fg=TKCOLOR.red)
            btn.bind("<ButtonRelease>", self.toggle_providers)

    def toggle_providers(self, event):
        btn = event.widget
        key = btn["text"]
        if self.data["providers"][key] is True:
            self.data["providers"][key] = False
            btn.configure(fg=TKCOLOR.red)
        elif self.data["providers"][key] is False:
            self.data["providers"][key] = True
            btn.configure(fg=TKCOLOR.green)
        raw_config.set_config(self.data)


class ShowContextMenu(tk.Frame):
    def __init__(self, parent):
        tk.Frame.__init__(self, parent)
        self.configure(bg=TKCOLOR.dark_grey)
        self.string_var = tk.StringVar()
        self.string_var.set(f"True")
        label = tk.Label(self, text="Context menu")
        label.configure(bg=TKCOLOR.dark_grey, fg=TKCOLOR.white_grey, font=TKFONT.cas8b)
        label.grid(row=0, column=0, sticky="w", padx=2, pady=2)
        self.clabel = tk.Label(self, textvariable=self.string_var)
        self.clabel.configure(bg=TKCOLOR.dark_grey, font=TKFONT.cas8b)
        self.clabel.grid(row=0, column=1, sticky="nsew", padx=2, pady=2)
        tk_tools.ColorPicker(self.string_var, self.clabel)
        btn_true = tk.Button(
            self,
            font=TKFONT.cas8b,
            text="True",
            bd=0,
            bg=TKCOLOR.light_black,
            fg=TKCOLOR.white_grey,
            activebackground=TKCOLOR.green,
            height=2,
            width=18,
        )
        btn_true.grid(row=0, column=3, pady=2)
        btn_false = tk.Button(
            self,
            font=TKFONT.cas8b,
            text="False",
            bd=0,
            bg=TKCOLOR.light_black,
            fg=TKCOLOR.white_grey,
            activebackground=TKCOLOR.red,
            height=2,
            width=18,
        )
        btn_false.grid(row=0, column=2, pady=2)
        btn_true.bind("<Enter>", self.enter_button)
        btn_true.bind("<Leave>", self.leave_button)
        btn_false.bind("<Enter>", self.enter_button)
        btn_false.bind("<Leave>", self.leave_button)
        tk_tools.row_col_minsize(self)

    def enter_button(self, event):
        btn = event.widget
        if btn["text"] == "True":
            btn.configure(bg=TKCOLOR.black, fg=TKCOLOR.green)
            btn.bind("<ButtonRelease>", self.button_set_true)
        if btn["text"] == "False":
            btn.configure(bg=TKCOLOR.black, fg=TKCOLOR.red)
            btn.bind("<ButtonRelease>", self.button_set_false)

    def leave_button(self, event):
        btn = event.widget
        btn.configure(bg=TKCOLOR.light_black, fg=TKCOLOR.white_grey)

    def button_set_true(self, event):
        self.string_var.set(f"True")
        tk_tools.ColorPicker(self.string_var, self.clabel)
        from subsearch.utils import raw_registry

        raw_registry.add_context_menu()
        raw_registry.write_all_valuex()

    def button_set_false(self, event):
        self.string_var.set(f"False")
        tk_tools.ColorPicker(self.string_var, self.clabel)
        from subsearch.utils import raw_registry

        raw_registry.remove_context_menu()


class FileExtensions(tk.Frame):
    def __init__(self, parent):
        tk.Frame.__init__(self, parent)
        self.configure(bg=TKCOLOR.dark_grey)
        self.data = raw_config.get_config()
        number_of_buttons = len(AVAIL_EXT.items())
        label = tk.Label(self, text="File extensions")
        label.configure(bg=TKCOLOR.dark_grey, fg=TKCOLOR.white_grey, font=TKFONT.cas8b)
        label.grid(row=0, column=0, sticky="w", padx=2, pady=2)
        self.rownum = 0
        self.colnum = 0
        for i, key in zip(range(number_of_buttons), AVAIL_EXT.keys()):
            self.rownum += 1
            if self.rownum > 4:
                self.rownum = 1
                self.colnum += 1

            btn = tk.Button(
                self,
                font=TKFONT.cas8b,
                text=key,
                bd=0,
                bg=TKCOLOR.light_black,
                fg=TKCOLOR.white_grey,
                activebackground=TKCOLOR.orange,
                height=2,
                width=18,
            )
            btn.grid(row=self.rownum, column=self.colnum, pady=2)
            if self.data["file_ext"][key] is False:
                btn.configure(fg=TKCOLOR.red)
            elif self.data["file_ext"][key] is True:
                btn.configure(fg=TKCOLOR.green)

            # btn.bind("<Button-1>", self.enter_button)
            btn.bind("<Enter>", self.enter_button)
            btn.bind("<Leave>", self.leave_button)
            tk_tools.row_col_minsize(self)

    def enter_button(self, event):
        btn = event.widget
        btn.configure(bg=TKCOLOR.black, fg=TKCOLOR.orange)
        btn.bind("<ButtonPress>", self.press_button)

    def leave_button(self, event):
        btn = event.widget
        key = btn["text"]
        if self.data["file_ext"][key] is True:
            btn.configure(bg=TKCOLOR.light_black, fg=TKCOLOR.green)
        if self.data["file_ext"][key] is False:
            btn.configure(bg=TKCOLOR.light_black, fg=TKCOLOR.red)

    def press_button(self, event):
        btn = event.widget
        key = btn["text"]
        if self.data["file_ext"][key] is True:
            btn.configure(bg=TKCOLOR.light_black, fg=TKCOLOR.green)
            btn.bind("<ButtonRelease>", self.toggle_ext)
        if self.data["file_ext"][key] is False:
            btn.configure(bg=TKCOLOR.light_black, fg=TKCOLOR.red)
            btn.bind("<ButtonRelease>", self.toggle_ext)

    def toggle_ext(self, event):
        btn = event.widget
        key = btn["text"]
        if self.data["file_ext"][key] is True:
            self.data["file_ext"][key] = False
            btn.configure(fg=TKCOLOR.red)
        elif self.data["file_ext"][key] is False:
            self.data["file_ext"][key] = True
            btn.configure(fg=TKCOLOR.green)
        raw_config.set_config(self.data)

        self.update_registry()

    def update_registry(self):
        from subsearch.utils import raw_registry

        raw_registry.write_all_valuex()


class ShowContextMenuIcon(tk.Frame):
    def __init__(self, parent):
        tk.Frame.__init__(self, parent)
        self.configure(bg=TKCOLOR.dark_grey)
        self.string_var = tk.StringVar()
        self.string_var.set(f"{CONTEXT_MENU_ICON}")
        label = tk.Label(self, text="Context menu icon")
        label.configure(bg=TKCOLOR.dark_grey, fg=TKCOLOR.white_grey, font=TKFONT.cas8b)
        label.grid(row=0, column=0, sticky="w", padx=2, pady=2)
        self.clabel = tk.Label(self, textvariable=self.string_var)
        self.clabel.configure(bg=TKCOLOR.dark_grey, font=TKFONT.cas8b)
        self.clabel.grid(row=0, column=1, sticky="nsew", padx=2, pady=2)
        tk_tools.ColorPicker(self.string_var, self.clabel)

        btn_true = tk.Button(
            self,
            font=TKFONT.cas8b,
            text="True",
            bd=0,
            bg=TKCOLOR.light_black,
            fg=TKCOLOR.white_grey,
            activebackground=TKCOLOR.green,
            height=2,
            width=18,
        )
        btn_true.grid(row=0, column=3, pady=2)
        btn_false = tk.Button(
            self,
            font=TKFONT.cas8b,
            text="False",
            bd=0,
            bg=TKCOLOR.light_black,
            fg=TKCOLOR.white_grey,
            activebackground=TKCOLOR.red,
            height=2,
            width=18,
        )
        btn_false.grid(row=0, column=2, pady=2)
        btn_true.bind("<Enter>", self.enter_button)
        btn_true.bind("<Leave>", self.leave_button)
        btn_false.bind("<Enter>", self.enter_button)
        btn_false.bind("<Leave>", self.leave_button)
        tk_tools.row_col_minsize(self)

    def enter_button(self, event):
        btn = event.widget
        if btn["text"] == "True":
            btn.configure(bg=TKCOLOR.black, fg=TKCOLOR.green)
            btn.bind("<ButtonRelease>", self.button_set_true)
        if btn["text"] == "False":
            btn.configure(bg=TKCOLOR.black, fg=TKCOLOR.red)
            btn.bind("<ButtonRelease>", self.button_set_false)

    def leave_button(self, event):
        btn = event.widget
        btn.configure(bg=TKCOLOR.light_black, fg=TKCOLOR.white_grey)

    def button_set_true(self, event):
        self.string_var.set(f"True")
        tk_tools.ColorPicker(self.string_var, self.clabel)
        raw_config.set_config_key_value("context_menu_icon", True)
        from subsearch.utils import raw_registry

        raw_registry.write_valuex("icon")

    def button_set_false(self, event):
        self.string_var.set(f"False")
        tk_tools.ColorPicker(self.string_var, self.clabel)
        raw_config.set_config_key_value("context_menu_icon", False)
        from subsearch.utils import raw_registry

        raw_registry.write_valuex("icon")


class ShowDownloadWindow(tk.Frame):
    def __init__(self, parent):
        tk.Frame.__init__(self, parent)
        self.configure(bg=TKCOLOR.dark_grey)
        self.string_var = tk.StringVar()
        self.string_var.set(f"{DL_WINDOW}")
        label = tk.Label(self, text="Download window")
        label.configure(bg=TKCOLOR.dark_grey, fg=TKCOLOR.white_grey, font=TKFONT.cas8b)
        label.grid(row=0, column=0, sticky="w", padx=2, pady=2)
        self.clabel = tk.Label(self, textvariable=self.string_var)
        self.clabel.configure(bg=TKCOLOR.dark_grey, font=TKFONT.cas8b)
        self.clabel.grid(row=0, column=1, sticky="nsew", padx=2, pady=2)
        tk_tools.ColorPicker(self.string_var, self.clabel)
        btn_true = tk.Button(
            self,
            font=TKFONT.cas8b,
            text="True",
            bd=0,
            bg=TKCOLOR.light_black,
            fg=TKCOLOR.white_grey,
            activebackground=TKCOLOR.green,
            height=2,
            width=18,
        )
        btn_true.grid(row=0, column=3, pady=2)
        btn_false = tk.Button(
            self,
            font=TKFONT.cas8b,
            text="False",
            bd=0,
            bg=TKCOLOR.light_black,
            fg=TKCOLOR.white_grey,
            activebackground=TKCOLOR.red,
            height=2,
            width=18,
        )
        btn_false.grid(row=0, column=2, pady=2)
        btn_true.bind("<Enter>", self.enter_button)
        btn_true.bind("<Leave>", self.leave_button)
        btn_false.bind("<Enter>", self.enter_button)
        btn_false.bind("<Leave>", self.leave_button)
        tk_tools.row_col_minsize(self)

    def enter_button(self, event):
        btn = event.widget
        if btn["text"] == "True":
            btn.configure(bg=TKCOLOR.black, fg=TKCOLOR.green)
            btn.bind("<ButtonRelease>", self.button_set_true)
        if btn["text"] == "False":
            btn.configure(bg=TKCOLOR.black, fg=TKCOLOR.red)
            btn.bind("<ButtonRelease>", self.button_set_false)

    def leave_button(self, event):
        btn = event.widget
        btn.configure(bg=TKCOLOR.light_black, fg=TKCOLOR.white_grey)

    def button_set_true(self, event):
        self.string_var.set(f"True")
        tk_tools.ColorPicker(self.string_var, self.clabel)
        raw_config.set_config_key_value("show_download_window", True)

    def button_set_false(self, event):
        self.string_var.set(f"False")
        tk_tools.ColorPicker(self.string_var, self.clabel)
        raw_config.set_config_key_value("show_download_window", False)


class ShowTerminalOnSearch(tk.Frame):
    def __init__(self, parent):
        tk.Frame.__init__(self, parent)
        self.configure(bg=TKCOLOR.dark_grey)
        self.string_var = tk.StringVar()
        self.string_var.set(f"{SHOW_TERMINAL}")
        label = tk.Label(self, text="Terminal on search")
        label.configure(bg=TKCOLOR.dark_grey, fg=TKCOLOR.white_grey, font=TKFONT.cas8b)
        label.grid(row=0, column=0, sticky="w", padx=2, pady=2)
        self.clabel = tk.Label(self, textvariable=self.string_var)
        self.clabel.configure(bg=TKCOLOR.dark_grey, font=TKFONT.cas8b)
        self.clabel.grid(row=0, column=1, sticky="nsew", padx=2, pady=2)
        tk_tools.ColorPicker(self.string_var, self.clabel)
        if current_user.check_is_exe() is False:
            btn_true = tk.Button(
                self,
                font=TKFONT.cas8b,
                text="True",
                bd=0,
                bg=TKCOLOR.light_black,
                fg=TKCOLOR.white_grey,
                activebackground=TKCOLOR.green,
                height=2,
                width=18,
            )
            btn_true.grid(row=0, column=3, pady=2)
            btn_false = tk.Button(
                self,
                font=TKFONT.cas8b,
                text="False",
                bd=0,
                bg=TKCOLOR.light_black,
                fg=TKCOLOR.white_grey,
                activebackground=TKCOLOR.red,
                height=2,
                width=18,
            )
            btn_false.grid(row=0, column=2, pady=2)
            btn_true.bind("<Enter>", self.enter_button)
            btn_true.bind("<Leave>", self.leave_button)
            btn_false.bind("<Enter>", self.enter_button)
            btn_false.bind("<Leave>", self.leave_button)
            tk_tools.row_col_minsize(self)

    def enter_button(self, event):
        btn = event.widget
        if btn["text"] == "True":
            btn.configure(bg=TKCOLOR.black, fg=TKCOLOR.green)
            btn.bind("<ButtonRelease>", self.button_set_true)
        if btn["text"] == "False":
            btn.configure(bg=TKCOLOR.black, fg=TKCOLOR.red)
            btn.bind("<ButtonRelease>", self.button_set_false)

    def leave_button(self, event):
        btn = event.widget
        btn.configure(bg=TKCOLOR.light_black, fg=TKCOLOR.white_grey)

    def button_set_true(self, event):
        self.string_var.set(f"True")
        tk_tools.ColorPicker(self.string_var, self.clabel)
        raw_config.set_config_key_value("show_terminal", True)
        raw_registry.write_valuex("command")

    def button_set_false(self, event):
        self.string_var.set(f"False")
        tk_tools.ColorPicker(self.string_var, self.clabel)
        raw_config.set_config_key_value("show_terminal", False)
        raw_registry.write_valuex("command")


class CheckForUpdates(tk.Frame):
    def __init__(self, parent):
        tk.Frame.__init__(self, parent)
        self.configure(bg=TKCOLOR.dark_grey)
        self.string_var = tk.StringVar()
        self.string_var.set(f"")
        label = tk.Label(self, text=f"")
        label.configure(bg=TKCOLOR.dark_grey, fg=TKCOLOR.white_grey, font=TKFONT.cas8b)
        label.grid(row=0, column=0, sticky="w", padx=2, pady=2)
        self.clabel = tk.Label(self, textvariable=self.string_var)
        self.clabel.configure(bg=TKCOLOR.dark_grey, fg=TKCOLOR.yellow, font=TKFONT.cas8b)
        self.clabel.grid(row=0, column=1, sticky="nsew", padx=2, pady=2)
        btn_check = tk.Button(
            self,
            font=TKFONT.cas8b,
            text="Check for updates",
            bd=0,
            bg=TKCOLOR.light_black,
            fg=TKCOLOR.white_grey,
            activebackground=TKCOLOR.green,
            height=2,
            width=18,
        )
        btn_check.grid(row=0, column=3, pady=2)
        self.btn_download = tk.Button(
            self,
            font=TKFONT.cas8b,
            text="",
            bd=0,
            bg=TKCOLOR.light_black,
            fg=TKCOLOR.white_grey,
            activebackground=TKCOLOR.red,
            height=2,
            width=18,
        )
        self.btn_download.grid(row=0, column=2, pady=2)
        btn_check.bind("<Enter>", self.enter_button)
        btn_check.bind("<Leave>", self.leave_button)
        self.btn_download.bind("<Enter>", self.enter_button)
        self.btn_download.bind("<Leave>", self.leave_button)
        tk_tools.row_col_minsize(self)

    def enter_button(self, event):
        btn = event.widget
        if btn["text"] == "Check for updates":
            btn.configure(bg=TKCOLOR.black, fg=TKCOLOR.green)
            btn.bind("<ButtonRelease>", self.button_check)
        if btn["text"].startswith("Get"):
            btn.configure(bg=TKCOLOR.black, fg=TKCOLOR.red)
            btn.bind("<ButtonRelease>", self.button_download)

    def leave_button(self, event):
        btn = event.widget
        btn.configure(bg=TKCOLOR.light_black, fg=TKCOLOR.white_grey)

    def button_check(self, event):
        self.string_var.set(f"Looking...")
        new_repo_avail, repo_is_prerelease = updates.is_new_version_avail()
        latest_version = updates.get_latest_version()
        if new_repo_avail and repo_is_prerelease:
            if "-rc" in latest_version:
                self.string_var.set(f"Release candidate")
            elif "-alpha" in latest_version:
                self.string_var.set(f"Alpha release")
            elif "-beta" in latest_version:
                self.string_var.set(f"Beta release")
        elif new_repo_avail and repo_is_prerelease is False:
            self.string_var.set(f"Stable release")
        else:
            self.string_var.set(f"Up to date")

        if new_repo_avail:
            self.btn_download.configure(text=f"Get v{latest_version}")

    def button_download(self, event):
        webbrowser.open("https://github.com/vagabondHustler/SubSearch/releases")


class Footer(tk.Frame):
    def __init__(self, parent):
        tk.Frame.__init__(self, parent)
        self.configure(bg=TKCOLOR.mid_grey_black, width=TKWINDOW.width - 4, height=80)
        self.active_window = "language"

        self.language_grey_path = tk_tools.buttons("language_grey.png")
        self.language_silver_path = tk_tools.buttons("language_silver.png")
        self.language_white_path = tk_tools.buttons("language_white.png")
        self.language_grey_png = tk.PhotoImage(file=self.language_grey_path)
        self.language_silver_png = tk.PhotoImage(file=self.language_silver_path)
        self.language_white_png = tk.PhotoImage(file=self.language_white_path)
        self.language = tk.Canvas(
            self,
            width=54,
            height=54,
            bg=TKCOLOR.mid_grey_black,
            highlightthickness=0,
        )
        self.language.place(relx=0.2, rely=0.5, anchor="center")
        self.language.bind("<Enter>", self.enter_language)
        self.language.bind("<Leave>", self.leave_language)
        self.update_img(self.language, self.language_grey_png)

        self.search_grey_path = tk_tools.buttons("search_grey.png")
        self.search_silver_path = tk_tools.buttons("search_silver.png")
        self.search_white_path = tk_tools.buttons("search_white.png")
        self.search_grey_png = tk.PhotoImage(file=self.search_grey_path)
        self.search_silver_png = tk.PhotoImage(file=self.search_silver_path)
        self.search_white_png = tk.PhotoImage(file=self.search_white_path)
        self.search = tk.Canvas(
            self,
            width=54,
            height=54,
            bg=TKCOLOR.mid_grey_black,
            highlightthickness=0,
        )
        self.search.place(relx=0.5, rely=0.5, anchor="center")
        self.search.bind("<Enter>", self.enter_search)
        self.search.bind("<Leave>", self.leave_search)
        self.update_img(self.search, self.search_grey_png)

        self.settings_grey_path = tk_tools.buttons("settings_grey.png")
        self.settings_silver_path = tk_tools.buttons("settings_silver.png")
        self.settings_white_path = tk_tools.buttons("settings_white.png")
        self.settings_grey_png = tk.PhotoImage(file=self.settings_grey_path)
        self.settings_silver_png = tk.PhotoImage(file=self.settings_silver_path)
        self.settings_white_png = tk.PhotoImage(file=self.settings_white_path)
        self.settings = tk.Canvas(
            self,
            width=54,
            height=54,
            bg=TKCOLOR.mid_grey_black,
            highlightthickness=0,
        )
        self.settings.place(relx=0.8, rely=0.5, anchor="center")
        self.settings.bind("<Enter>", self.enter_settings)
        self.settings.bind("<Leave>", self.leave_settings)
        self.update_img(self.settings, self.settings_grey_png)
        tk_tools.row_col_minsize(self)

        self.activate_window()

    def release_language(self, event):
        self.active_window = "language"
        self.activate_window()

    def release_search(self, event):
        self.active_window = "search"
        self.activate_window()

    def release_settings(self, event):
        self.active_window = "settings"
        self.activate_window()

    def press_language(self, event):
        self.language.bind("<ButtonRelease>", self.release_language)
        self.update_img(self.language, self.language_white_png, y=20)

    def press_search(self, event):
        self.search.bind("<ButtonRelease>", self.release_search)
        self.update_img(self.search, self.search_white_png, y=20)

    def press_settings(self, event):
        self.settings.bind("<ButtonRelease>", self.release_settings)
        self.update_img(self.settings, self.settings_white_png, y=20)

    def activate_window(self):
        # language window
        if self.active_window == "language":
            language_window.place(relx=0.5, rely=0.5, anchor="center")
            self.update_img(self.language, self.language_white_png)
        else:
            language_window.place(relx=1, rely=0.5, anchor="nw")
            self.update_img(self.language, self.language_grey_png)

        # search window
        if self.active_window == "search":
            search_window.place(relx=0.5, rely=0.5, anchor="center")
            self.update_img(self.search, self.search_white_png)
        else:
            search_window.place(relx=1, rely=0.5, anchor="nw")
            self.update_img(self.search, self.search_grey_png)

        # settings window
        if self.active_window == "settings":
            settings_window.place(relx=0.5, rely=0.5, anchor="center")
            self.update_img(self.settings, self.settings_white_png)
        else:
            settings_window.place(relx=1, rely=0.5, anchor="nw")
            self.update_img(self.settings, self.settings_grey_png)

    def enter_search(self, event):
        self.search.bind("<ButtonPress>", self.press_search)
        self.update_img(self.search, self.search_silver_png, y=25)

    def leave_search(self, event):
        self.search.unbind("<ButtonPress>")
        self.activate_window()

    def enter_settings(self, event):
        self.settings.bind("<ButtonPress>", self.press_settings)
        self.update_img(self.settings, self.settings_silver_png, y=25)

    def leave_settings(self, event):
        self.settings.unbind("<ButtonPress>")
        self.activate_window()

    def enter_language(self, event):
        self.language.bind("<ButtonPress>", self.press_language)
        self.update_img(self.language, self.language_silver_png, y=25)

    def leave_language(self, event):
        self.language.unbind("<ButtonPress>")
        self.activate_window()

    def update_img(self, canvas, img, x=27, y=27):
        canvas.delete("all")
        canvas.create_image(x, y, image=img)
        canvas.photoimage = img


class LanguageWindow(tk.Frame):
    def __init__(self, parent):
        tk.Frame.__init__(self, parent)
        self.configure(bg=TKCOLOR.dark_grey)
        SelectLanguage(self).pack(anchor="center")


class SearchWindow(tk.Frame):
    def __init__(self, parent):
        tk.Frame.__init__(self, parent)
        self.configure(bg=TKCOLOR.dark_grey)
        Providers(self).pack(anchor="center")
        HearingImparedSubs(self).pack(anchor="center")
        SearchThreshold(self).pack(anchor="center")


class SettingsWindow(tk.Frame):
    def __init__(self, parent):
        tk.Frame.__init__(self, parent)
        self.configure(bg=TKCOLOR.dark_grey)
        FileExtensions(self).pack(anchor="center")
        tk.Frame(self, height=80, bg=TKCOLOR.dark_grey).pack(anchor="center", expand=True)
        ShowContextMenu(self).pack(anchor="center")
        ShowContextMenuIcon(self).pack(anchor="center")
        ShowDownloadWindow(self).pack(anchor="center")
        if current_user.check_is_exe() is False:
            ShowTerminalOnSearch(self).pack(anchor="center")
        tk.Frame(self, height=20, bg=TKCOLOR.dark_grey).pack(anchor="center", expand=True)
        CheckForUpdates(self).pack(anchor="center")


def show_widget():
    global base_root, language_window, search_window, settings_window
    if current_user.got_key() is False:
        raw_config.set_default_json()
        raw_registry.add_context_menu()

    base_root = base_root.main()
    content = tk.Frame(base_root, bg=TKCOLOR.dark_grey, width=TKWINDOW.width - 4, height=TKWINDOW.height - 120)
    content.place(x=2, y=40)
    language_window = LanguageWindow(content)
    search_window = SearchWindow(content)
    settings_window = SettingsWindow(content)
    footer = Footer(base_root)
    footer.place(x=2, y=TKWINDOW.height - 82)

    base_root.mainloop()



import re
from typing import Union

from subsearch.data import __video_directory__
from subsearch.providers import generic
from subsearch.providers.generic import BaseChecks, BaseProvider, DownloadData
from subsearch.utils import log, string_parser
from subsearch.utils.raw_config import UserParameters
from subsearch.utils.string_parser import FileSearchParameters


class OpenSubtitles(BaseProvider):
    def __init__(self, parameters: FileSearchParameters, user_parameters: UserParameters):
        BaseProvider.__init__(self, parameters, user_parameters)
        self.scrape = OpenSubtitlesScrape()
        self.check = BaseChecks()

    def parse_hash(self):
        to_be_downloaded: list[str] | None = self.scrape.hash(self.url_hash, self.current_language, self.hearing_impaired)
        if to_be_downloaded is None and self.series:
            log.output(f"No TV-series found matching hash {self.file_hash}")
            log.output(f"Done with tasks\n")
            return None
        if to_be_downloaded is None:
            log.output(f"No movies found matching hash {self.file_hash}")
            log.output(f"Done with tasks\n")
            return None

        download_info = []
        log.output(f"Preparing  hash {self.file_hash} for download")
        tbd_lenght = len(to_be_downloaded)
        for zip_idx, zip_url in enumerate(to_be_downloaded, start=1):
            zip_fp = f"{__video_directory__}\\__subsearch__opensubtitles_{zip_idx}.zip"
            data = DownloadData(file_path=zip_fp, url=zip_url, idx_num=zip_idx, idx_lenght=tbd_lenght)
            download_info.append(data)
        log.output(f"Done with tasks\n")
        return download_info

    def parse_rss(self):
        to_be_sorted = []
        results = self.scrape.rss_subtitles(self.url_opensubtitles)
        to_be_downloaded: dict[str, str] = {}
        to_be_sorted: list[tuple[int, str, str]] = []
        for key, value in results.items():
            number = string_parser.get_pct_value(key, self.release)
            log.output(f"[Found]: {key}")
            lenght_str = sum(1 for char in f"[{number}% match]:")
            formatting_spaces = " " * lenght_str
            _name = f"[{number}% match]: {key}"
            _url = f"{formatting_spaces} {value}"
            to_be_sorted_value = number, _name, _url
            to_be_sorted.append(to_be_sorted_value)
            if self.check.is_threshold_met(key, self.title, self.season, self.episode, self.series, number, self.pct):
                to_be_downloaded[key] = value if key not in to_be_downloaded else None
        self._sorted_list = generic.log_and_sort_list("opensubtitles", to_be_sorted, self.pct)
        # exit if no subtitles found
        if len(to_be_downloaded) == 0:
            log.output(f"No subtitles to download for {self.release}")
            log.output("Done with tasks\n")
            return None

        download_info = []
        tbd_lenght = len(to_be_downloaded)
        for zip_idx, (zip_name, zip_url) in enumerate(to_be_downloaded.items(), start=1):
            zip_fp = f"{__video_directory__}\\__subsearch__opensubtitles_{zip_idx}.zip"
            data = DownloadData(name=zip_name, file_path=zip_fp, url=zip_url, idx_num=zip_idx, idx_lenght=tbd_lenght)
            download_info.append(data)
        log.output("Done with tasks\n")
        return download_info

    def sorted_list(self):
        return self._sorted_list


class OpenSubtitlesScrape:
    def __init__(self):
        self.check = BaseChecks()

    def rss_subtitles(self, url: str):
        subtitles: dict[str, str] = {}
        doc = generic.get_lxml_doc(url, "xml")
        items = doc.find_all("item")
        for item in items:
            dl_url = item.enclosure["url"]
            released_as = item.description.text.strip()
            release_name = re.findall("^.*?: (.*?);", released_as)[0]  # https://regex101.com/r/LWAmJK/1
            subtitles[release_name] = dl_url
        return subtitles

    def hash(self, url: str, current_language: str, hearing_impaired: Union[str, bool]) -> list[str] | None:
        download_url: list[str] = []
        doc = generic.get_lxml_doc(url, "lxml")
        doc_results = doc.find("table", id="search_results")
        if doc_results is None:
            return None
        tr_name = doc_results.find_all("tr", id=lambda value: value and value.startswith("name"))
        for item in tr_name:
            tl = [a["title"] for a in item.find_all("a", title=current_language)]
            if current_language not in tl:
                continue
            hi_site = item.find("img", alt="Subtitles for hearing impaired")
            if hi_site is not None and hearing_impaired is False:
                log.output(f"Found HI-subtitle but skipping, 'cus hearing impaired is set to '{hearing_impaired}'")
                continue
            if hi_site is None and hearing_impaired is True:
                log.output(f"Found nonHI-subtitle but skipping, 'cus hearing impaired is set to '{hearing_impaired}'")
                continue

            title_name = item.find("a", class_="bnone").text.replace("\n", "").replace("\t", "").replace("(", " (")
            log.output(f"{title_name} matched file hash")
            th = [
                a["href"]
                for a in item.find_all(
                    "a",
                    href=lambda value: value and value.startswith("/en/subtitleserve/sub/"),
                )
            ]
            th = th[0] if th is not None else None
            link = f"https://www.opensubtitles.org{th}"
            download_url.append(link) if th is not None else None
        if len(download_url) == 0:
            return None
        return download_url



import logging
import os
import sys
from datetime import datetime
from typing import Literal

from subsearch.data import __version__, __video_directory__
from subsearch.utils import raw_config
from subsearch.utils.raw_config import UserParameters
from subsearch.utils.string_parser import FileSearchParameters

CWD = os.getcwd()
if __video_directory__ is not None:
    CWD = __video_directory__


NOW = datetime.now()
DATE = NOW.strftime("%y%m%d")

logging.basicConfig(
    filename=f"{CWD}\\__subsearch__.log",
    filemode="w",
    level=logging.DEBUG,
    format="%(asctime)s - %(message)s",
    datefmt="%d-%b-%y %H:%M:%S",
)


def output(msg: str, to_terminal: bool = True) -> Literal[False] | None:
    # log and print message
    logging.info(msg)
    return print(msg) if to_terminal else False


def tprint(msg: str) -> None:
    root = logging.getLogger()
    root.setLevel(logging.DEBUG)
    handler = logging.StreamHandler(sys.stdout)
    handler.setLevel(logging.DEBUG)
    formatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
    handler.setFormatter(formatter)
    root.addHandler(handler)
    return logging.info(msg)


def parameters(param: FileSearchParameters, user_parameters: UserParameters) -> None:
    # log and print all the used parameters from video/directory-name
    languages = raw_config.get_config_key("languages")
    lang_code3 = languages[user_parameters.current_language]
    output(f"[PARAMETERS]                [VALUES]")
    output(f"subsearch:                  v{__version__} ")
    output(f"Language:                   {user_parameters.current_language}, {lang_code3}")
    output(f"Hearing impaired:           {user_parameters.hearing_impaired}")
    output(f"Title:                      {param.title}")
    output(f"Year:                       {param.year}")
    output(f"Season:                     {param.season}, {param.season_ordinal}")
    output(f"Episode:                    {param.episode}, {param.episode_ordinal}")
    output(f"Series:                     {param.series}")
    output(f"Release:                    {param.release}")
    output(f"Group:                      {param.group}")
    output(f"Match threshold:            {user_parameters.pct}%")
    output(f"File hash:                  {param.file_hash}")
    output(f"URL Subscene:               {param.url_subscene}")
    output(f"URL OpenSubtitles - rss:    {param.url_opensubtitles}")
    output(f"URL OpenSubtitles - hash:   {param.url_opensubtitles_hash}\n")



import ctypes
import os
import sys
import time

from subsearch.data import (
    __version__,
    __video_directory__,
    __video_name__,
    __video_path__,
)
from subsearch.gui import widget_config, widget_downloads
from subsearch.providers import opensubtitles, subscene
from subsearch.utils import (
    current_user,
    file_manager,
    log,
    raw_config,
    raw_registry,
    string_parser,
)

PACKAGEPATH = os.path.abspath(os.path.dirname(__file__))
HOMEPATH = os.path.dirname(PACKAGEPATH)
sys.path.append(HOMEPATH)
sys.path.append(PACKAGEPATH)


class Subsearch:
    def __init__(self) -> None:
        """
        Setup and gather all available parameters
        """
        self.start = time.perf_counter()
        ctypes.windll.kernel32.SetConsoleTitleW(f"SubSearch - {__version__}")
        if current_user.got_key() is False:
            raw_config.set_default_json()
            raw_registry.add_context_menu()
        self.show_terminal = False if current_user.check_is_exe() else raw_config.get_config_key("show_terminal")
        self.providers = raw_config.get_config_key("providers")
        self.file_exist = True if __video_name__ is not None else False
        self.current_language = raw_config.get_config_key("current_language")
        self.languages = raw_config.get_config_key("languages")
        self.hi = raw_config.get_config_key("hearing_impaired")
        self.pct = raw_config.get_config_key("percentage")
        self.show_dl_win = raw_config.get_config_key("show_download_window")
        self.user_parameters = raw_config.UserParameters(
            current_language=self.current_language,
            hearing_impaired=self.hi,
            pct=self.pct,
            show_dl_window=self.show_dl_win,
        )
        if self.file_exist:
            file_hash = file_manager.get_hash(__video_path__)
            self.parameters = string_parser.get_parameters(__video_name__, file_hash, self.current_language, self.languages)
            log.parameters(self.parameters, self.user_parameters)
            if " " in __video_name__:
                log.output("[Warning: Filename contains spaces]")
        if self.file_exist is False:
            widget_config.show_widget()

    def opensubtitles_scrape(self) -> None:
        """
        Scrape on opensubtitles by filehash
        """
        if self.languages[self.current_language] == "N/A":
            log.output("\n[Searching on opensubtitles]")
            log.output(f"{self.current_language} not supported on opensubtitles\n")
            self.opensubtitles_hash = None
            self.opensubtitles_rss = None
            self.opensubtitles_sorted_list = []
            return
        if self.file_exist and (self.providers["opensubtitles_hash"] or self.providers["opensubtitles_rss"]):
            _opensubtitles = opensubtitles.OpenSubtitles(self.parameters, self.user_parameters)
            if self.providers["opensubtitles_hash"]:
                log.output("\n[Searching on opensubtitles - hash]")
                self.opensubtitles_hash = _opensubtitles.parse_hash()
            if self.providers["opensubtitles_rss"]:
                log.output("\n[Searching on opensubtitles - rss]")
                self.opensubtitles_rss = _opensubtitles.parse_rss()
            self.opensubtitles_sorted_list = _opensubtitles.sorted_list()

    def subscene_scrape(self) -> None:
        """
        Scrape subscene from parsing filename of the video
        """
        if self.file_exist and self.providers["subscene"]:
            log.output("\n[Searching on subscene - title]")
            _subscene = subscene.Subscene(self.parameters, self.user_parameters)
            self.subscene = _subscene.parse()
            self.subscene_sorted_list = _subscene.sorted_list()
        else:
            self.subscene = None

    def process_files(self) -> None:
        """
        Download zip files containing the .srt files, extract, rename and clean up tmp files
        """
        os_hash_dls = 0
        os_rss_dls = 0
        ss_dls = 0
        if self.file_exist is False:
            return None

        if self.providers["opensubtitles_hash"] and self.opensubtitles_hash is not None:
            log.output("\n[Downloading from opensubtitles - hash]")
            for item in self.opensubtitles_hash:
                os_hash_dls = file_manager.download_subtitle(item)
                log.output("\n")

        if self.providers["opensubtitles_rss"] and self.opensubtitles_rss is not None:
            log.output("\n[Downloading from opensubtitles - rss]")
            for item in self.opensubtitles_rss:
                os_rss_dls = file_manager.download_subtitle(item)

        if self.providers["subscene"] and self.subscene is not None:
            log.output("\n[Downloading from subscene]")
            for item in self.subscene:
                ss_dls = file_manager.download_subtitle(item)

        total_dls = os_hash_dls + os_rss_dls + ss_dls
        if self.show_dl_win and total_dls == 0:
            not_downloaded = list(self.opensubtitles_sorted_list)
            not_downloaded.extend(x for x in self.subscene_sorted_list if x not in not_downloaded)
            not_downloaded.sort(key=lambda x: x[0], reverse=True)
            if len(not_downloaded) > 0:
                tmp_file = file_manager.write_not_downloaded_tmp(__video_directory__, not_downloaded)
                widget_downloads.show_widget()
                file_manager.clean_up(__video_directory__, tmp_file)

        log.output("\n[Procsessing files]")
        file_manager.extract_files(__video_directory__, ".zip")
        file_manager.clean_up(__video_directory__, ".zip")
        file_manager.clean_up(__video_directory__, ").nfo")
        file_manager.rename_best_match(f"{self.parameters.release}.srt", __video_directory__, ".srt")

    def end(self) -> None:
        """
        Stop pref counter, log elapsed time and keep the terminal open if show_terminal is True
        """
        elapsed = time.perf_counter() - self.start
        log.output(f"\nFinished in {elapsed} seconds")

        if self.show_terminal and current_user.check_is_exe() is False:
            try:
                input("Ctrl + c or Enter to exit")
            except KeyboardInterrupt:
                pass


def con() -> None:
    r"""
    Usages: subsearch [OPTIONS]

    Options:
        --settings                                  Open the GUI settings menu

        --registry-key [add, del]                   Edit the registry
                                                    add: adds the context menu  / replaces the context menu with default values
                                                    del: deletes the context menu
                                                    e.g: subsearch --registry-key add

        --help                                      Prints usage information
    """
    if sys.argv[-1].endswith("subsearch"):
        print(con.__doc__)
        return
    else:
        for num, arg in enumerate(sys.argv[1:], 1):
            if arg.startswith("--settings"):
                sys.argv.pop(num)  # pop settings argument
                widget_config.show_widget()
                break
            elif arg.startswith("--registry-key") or arg.startswith("--add-key"):
                if sys.argv[num + 1] == "add":
                    sys.argv.pop(num)  # pop registry argument
                    sys.argv.pop(num)  # pop add argument
                    raw_registry.add_context_menu()
                    break
                elif sys.argv[num + 1] == "del":
                    sys.argv.pop(num)  # pop registry argument
                    sys.argv.pop(num)  # pop del argument
                    raw_registry.remove_context_menu()
                    break
            elif arg.startswith("--help"):
                sys.argv.pop(num)  # pop help argument
                print(con.__doc__)
                break
            elif len(sys.argv[1:]) == num:
                print("Invalid argument")
                print(con.__doc__)


def main() -> None:
    for i in sys.argv:
        if i.startswith("--"):
            con()
            return None

    ss = Subsearch()
    ss.opensubtitles_scrape()
    ss.subscene_scrape()
    ss.process_files()
    ss.end()


if __name__ == "__main__":
    main()



import time
from typing import NamedTuple

import cloudscraper
from bs4 import BeautifulSoup
from bs4.element import Tag

from subsearch.utils import log
from subsearch.utils.exceptions import CaptchaError
from subsearch.utils.raw_config import UserParameters
from subsearch.utils.string_parser import FileSearchParameters

SCRAPER = cloudscraper.create_scraper(browser={"browser": "chrome", "platform": "android", "desktop": False})


def get_lxml_doc(url: str, features: str = "lxml") -> BeautifulSoup:
    source = SCRAPER.get(url)
    scontent = source.content
    doc = BeautifulSoup(scontent, features)
    return doc


def log_and_sort_list(_provider: str, list_of_tuples: list[tuple[int, str, str]], pct: int):
    list_of_tuples.sort(key=lambda x: x[0], reverse=True)
    log.output(f"\n[Sorted List from {_provider}]", False)
    hbd_printed = False
    hnbd_printed = False
    for i in list_of_tuples:
        name = i[1]
        url = i[2]
        if i[0] >= pct and not hbd_printed:
            log.output(f"--- Has been downloaded ---\n", False)
            hbd_printed = True
        if i[0] <= pct and not hnbd_printed:
            log.output(f"--- Has not been downloaded ---\n", False)
            hnbd_printed = True
        log.output(f"{name}", False)
        log.output(f"{url}\n", False)
    return list_of_tuples


class DownloadData(NamedTuple):
    name: str
    file_path: str
    url: str
    idx_num: int
    idx_lenght: int


class BaseProvider:
    """
    Base class for providers
    """

    def __init__(self, parameters: FileSearchParameters, user_parameters: UserParameters):
        # file parameters
        self.url_subscene = parameters.url_subscene
        self.url_opensubtitles = parameters.url_opensubtitles
        self.url_hash = parameters.url_opensubtitles_hash
        self.title = parameters.title
        self.year = parameters.year
        self.season = parameters.season
        self.season_ordinal = parameters.season_ordinal
        self.episode = parameters.episode
        self.episode_ordinal = parameters.episode_ordinal
        self.series = parameters.series
        self.release = parameters.release
        self.group = parameters.group
        self.file_hash = parameters.file_hash
        self.definitive_match = parameters.definitive_match
        # user parameters
        self.current_language = user_parameters.current_language
        self.hearing_impaired = user_parameters.hearing_impaired
        self.pct = user_parameters.pct
        self.show_download_window = user_parameters.show_dl_window
        # sorted_list
        self._sorted_list: list[DownloadData] = []


class BaseChecks:
    """
    Base class for checks
    """

    def __init__(self) -> None:
        pass

    def is_movie(self, key: str, title: str, year: int) -> bool:
        if key.lower() == f"{title} ({year})":
            log.output(f"Movie {key} found")
            return True
        return False

    def try_the_year_before(self, key: str, title: str, year: int) -> bool:
        # Some releases are released close to the next year. If so, the year might differ from the year in the title.
        # This function subtracts one year from the year in the title and checks if the release is in the list.
        if year == "N/A":
            return False
        _year = int(year) - 1
        the_year_before = f"{title} ({_year})"
        if key.lower().startswith(the_year_before):
            log.output(f"Movie {key} found")
            return True
        return False

    def is_series(self, key: str, title: str, season_ordinal: str, show_bool: bool) -> bool:
        if title and season_ordinal in key.lower() and show_bool:
            log.output(f"TV-Series {key} found")
            return True
        return False

    def is_threshold_met(
        self, key: str, title: str, season: str, episode: str, show_bool: bool, number: int, pct: int
    ) -> bool:
        if number >= pct or title and f"{season}{episode}" in key.lower() and show_bool:
            return True
        return False

    def is_subtitle_hearing_impaired(self, a1: Tag) -> bool | None:
        a1_parent = a1.parent
        class_a40 = a1_parent.find("td", class_="a40")  # non-hearing impaired
        class_a41 = a1_parent.find("td", class_="a41")  # hearing imparted
        if class_a40 is None:
            return True
        elif class_a41 is None:
            return False
        return None

    def to_many_requests(body: Tag):
        time.sleep(1)
        log.output(f"ToManyRequestsWarning: To many requests, sleept for 1 second")

    def is_captcha(self, doc: Tag):
        tag_h2 = doc.find("h2", text="Why do I have to complete a CAPTCHA?")
        if tag_h2 is not None:
            raise CaptchaError("Captcha protection detected. Please try again later.")



import os
import shutil
import struct
import zipfile

import cloudscraper

from subsearch.providers.generic import DownloadData
from subsearch.utils import log, string_parser

SCRAPER = cloudscraper.create_scraper(browser={"browser": "chrome", "platform": "android", "desktop": False})


def download_subtitle(data: DownloadData) -> int:
    log.output(f"Downloading: {data.idx_num}/{data.idx_lenght}: {data.name}")
    r = SCRAPER.get(data.url, stream=True)
    with open(data.file_path, "wb") as fd:
        for chunk in r.iter_content(chunk_size=1024):
            fd.write(chunk)

    return data.idx_num


def extract_files(cwd: str, extension: str) -> None:
    # extract all zip file in said directory
    subs_folder = os.path.join(cwd, "subs")
    if not os.path.exists(subs_folder):
        os.mkdir(subs_folder)
    for file in os.listdir(cwd):
        if file.startswith("__subsearch__") and file.endswith(extension):
            log.output(f"Extracting: {file} -> ..\\subs\\{file}")
            filename = os.path.join(cwd, file)
            zip_ref = zipfile.ZipFile(filename)
            zip_ref.extractall(subs_folder)
            zip_ref.close()


def rename_best_match(release_name: str, cwd: str, extension: str) -> None:
    # rename a .srts to the same as video release name
    higest_value = (0, "")
    subs_folder = os.path.join(cwd, "subs")
    for file in os.listdir(subs_folder):
        if file.endswith(extension):
            value = string_parser.get_pct_value(file, release_name)
            if value >= higest_value[0]:
                higest_value = value, file

    file_to_rename = higest_value[1]
    if file_to_rename.endswith(extension):
        old_name_src = os.path.join(subs_folder, file_to_rename)
        new_name_dst = os.path.join(subs_folder, release_name)
        log.output(f"Renaming: {file_to_rename } -> {release_name}")
        os.rename(old_name_src, new_name_dst)
        move_src = new_name_dst
        move_dst = os.path.join(cwd, release_name)
        log.output(f"Moving: {release_name} -> {cwd}")
        shutil.move(move_src, move_dst)


# remove .zips
def clean_up(cwd: str, extension: str) -> None:
    for file in os.listdir(cwd):
        if file.startswith("__subsearch__") and file.endswith(extension):
            log.output(f"Removing: {file}")
            file_path = os.path.join(cwd, file)
            os.remove(file_path)


def write_not_downloaded_tmp(dst: str, not_downloaded: list) -> str:
    file_dst = f"{dst}\\__subsearch__dl_data.tmp"
    with open(file_dst, "w", encoding="utf8") as f:
        for i in range(len(not_downloaded)):
            name, _link = not_downloaded[i][1], not_downloaded[i][2]
            link = _link.replace(" ", "")
            f.writelines(f"{name} {link}")
            f.write("\n")
    return file_dst


# get file hash
def get_hash(file_path: str) -> str | None:
    try:
        longlongformat = "<q"  # little-endian long long
        bytesize = struct.calcsize(longlongformat)
        with open(file_path, "rb") as f:
            filesize = os.path.getsize(file_path)
            hash = filesize
            if filesize < 65536 * 2:
                log.output(f"SizeError: filesize is {filesize} bytes", False)
                return None
            n1 = 65536 // bytesize
            for _x in range(n1):
                buffer = f.read(bytesize)
                (l_value,) = struct.unpack(longlongformat, buffer)
                hash += l_value
                hash = hash & 0xFFFFFFFFFFFFFFFF  # to remain as 64bit number
            f.seek(max(0, filesize - 65536), 0)
            n2 = 65536 // bytesize
            for _x in range(n2):
                buffer = f.read(bytesize)
                (l_value,) = struct.unpack(longlongformat, buffer)
                hash += l_value
                hash = hash & 0xFFFFFFFFFFFFFFFF

        returnedhash = "%016x" % hash
        return returnedhash

    except IOError:
        log.output("IOError")
        return None



from bs4 import BeautifulSoup

from subsearch.data import __video_directory__
from subsearch.providers import generic
from subsearch.providers.generic import SCRAPER, BaseChecks, BaseProvider, DownloadData
from subsearch.utils import log, string_parser
from subsearch.utils.raw_config import UserParameters
from subsearch.utils.string_parser import FileSearchParameters


class Subscene(BaseProvider):
    def __init__(self, parameters: FileSearchParameters, user_parameters: UserParameters):
        BaseProvider.__init__(self, parameters, user_parameters)
        self.scrape = SubsceneScrape()
        self.check = BaseChecks()

    def parse(self):
        to_be_scraped: list[str] = []
        definitive_match = self.url_subscene.rsplit("query=", 1)[-1].replace("%20", " ")
        title_keys = self.scrape.title(self.url_subscene, definitive_match)
        for key, value in title_keys.items():
            if self.check.is_movie(key, self.title, self.year):
                to_be_scraped.append(value) if value not in (to_be_scraped) else None
            if self.check.try_the_year_before(key, self.title, self.year):
                to_be_scraped.append(value) if value not in (to_be_scraped) else None
            if self.check.is_series(key, self.title, self.season_ordinal, self.series):
                to_be_scraped.append(value) if value not in (to_be_scraped) else None
        log.output("Done with task\n") if len(to_be_scraped) > 0 else None

        # exit if no titles found
        if len(to_be_scraped) == 0:
            if self.series:
                log.output(f"\nNo TV-series found matching {self.title}")
            else:
                log.output(f"\nNo movies found matching {self.title}")
            return None

        # search title for subtitles
        to_be_downloaded: dict[str, str] = {}
        to_be_sorted: list[tuple[int, str, str]] = []
        while len(to_be_scraped) > 0:
            for subtitle_url in to_be_scraped:
                log.output(f"[Searching for on subscene - subtitle]")
                sub_keys = self.scrape.subtitle(self.current_language, self.hearing_impaired, subtitle_url)
                break
            for key, value in sub_keys.items():
                number = string_parser.get_pct_value(key, self.release)
                log.output(f"[Found]: {key}")
                lenght_str = sum(1 for char in f"[{number}% match]:")
                formatting_spaces = " " * lenght_str
                _name = f"[{number}% match]: {key}"
                _url = f"{formatting_spaces} {value}"
                to_be_sorted_value = number, _name, _url
                to_be_sorted.append(to_be_sorted_value)
                if self.check.is_threshold_met(key, self.title, self.season, self.episode, self.series, number, self.pct):
                    to_be_downloaded[key] = value if key not in to_be_downloaded else None
            to_be_scraped.pop(0) if len(to_be_scraped) > 0 else None
            self._sorted_list = generic.log_and_sort_list("subscene", to_be_sorted, self.pct)
        # exit if no subtitles found
        if len(to_be_downloaded) == 0:
            log.output(f"No subtitles to download for {self.release}")
            log.output("Done with tasks\n")
            return None

        download_info = []
        tbd_lenght = len(to_be_downloaded)
        for zip_idx, (zip_name, _zip_url) in enumerate(to_be_downloaded.items(), start=1):
            zip_url = self.scrape.download_url(_zip_url)
            zip_fp = f"{__video_directory__}\\__subsearch__subscene_{zip_idx}.zip"
            data = DownloadData(name=zip_name, file_path=zip_fp, url=zip_url, idx_num=zip_idx, idx_lenght=tbd_lenght)
            download_info.append(data)
        log.output("Done with tasks\n")
        return download_info

    def sorted_list(self):
        return self._sorted_list


class SubsceneScrape:
    def __init__(self):
        self.check = BaseChecks()

    def title(self, url: str, definitive_match: str) -> dict[str, str]:
        titles: dict[str, str] = {}
        doc = generic.get_lxml_doc(url)
        tag_div = doc.find("div", class_="search-result")
        self.check.is_captcha(tag_div)
        tag_div = tag_div.find_all("div", class_="title")
        for class_title in tag_div:
            title_name = class_title.contents[1].contents[0].strip().replace(":", "")
            title_url = class_title.contents[1].attrs["href"]
            titles[title_name] = f"https://subscene.com{title_url}"
            if title_name.lower() == definitive_match:
                break
        return titles

    def subtitle(self, current_language: str, hearing_impaired: bool | str, url: str) -> dict[str, str]:
        subtitles: dict[str, str] = {}
        doc = generic.get_lxml_doc(url)
        tag_tbody = doc.find("tbody")
        while tag_tbody is None:
            self.check.to_many_requests(tag_tbody)
            tag_tbody = doc.find("tbody")
        tag_td = tag_tbody.find_all("td", class_="a1")
        for class_a1 in tag_td:
            sub_hi = self.check.is_subtitle_hearing_impaired(class_a1)
            subtitle_language = class_a1.contents[1].contents[1].contents[0].strip()
            if hearing_impaired != "Both" and hearing_impaired != sub_hi:
                continue
            if current_language.lower() != subtitle_language.lower():
                continue
            release_name = class_a1.contents[1].contents[3].string.strip()
            release_name = release_name.replace(" ", ".")
            subtitle_url = class_a1.contents[1].attrs["href"]
            subtitles[release_name] = f"https://subscene.com{subtitle_url}"
        return subtitles

    def download_url(self, url: str) -> str:
        source = SCRAPER.get(url).text
        doc = BeautifulSoup(source, "lxml")
        button_url = [dl["href"] for dl in doc.find_all("a", href=True, id="downloadButton")]
        download_url = f"https://subscene.com/{button_url[0]}"
        return download_url



import json
import os
from typing import Any, NamedTuple, Union

from subsearch.data import __data__


# update config.json
def set_config_key_value(key: str, value: Union[str, int, bool]) -> None:
    config_file = f"{__data__}\\config.json"
    with open(config_file, "r+", encoding="utf-8") as f:
        data = json.load(f)
        data[key] = value
        f.seek(0)
        json.dump(data, f, indent=4)
        f.truncate()


def set_config(data):
    config_file = os.path.join(__data__, "config.json")
    with open(config_file, "w") as f:
        f.seek(0)
        json.dump(data, f, indent=4)
        f.truncate()


def get_config() -> Any:
    config_file = f"{__data__}\\config.json"
    with open(config_file, encoding="utf-8") as file:
        data = json.load(file)

    return data


# get said value(s) from config.json
def get_config_key(key: str) -> Any:
    """
    Get values of keys in config.json

    Args:
        key (str):

        - User settings

        "hearing_impaired: bool | str", "percentage: int"

        - GUI stuff

        "cm_icon: bool", "show_download_window: bool",
        "show_terminal: bool", "file_ext: dict"

    Returns:
        Any: value
    """
    config_json_dict = {
        "current_language": get_config()["current_language"],
        "languages": get_config()["languages"],
        "hearing_impaired": get_config()["hearing_impaired"],
        "percentage": get_config()["percentage"],
        "context_menu_icon": get_config()["context_menu_icon"],
        "show_download_window": get_config()["show_download_window"],
        "show_terminal": get_config()["show_terminal"],
        "file_ext": get_config()["file_ext"],
        "providers": get_config()["providers"],
    }
    return config_json_dict[f"{key}"]


def set_default_json() -> None:
    # set default config.json values
    data = get_config()
    data["current_language"] = "English"
    data["hearing_impaired"] = "Both"
    data["percentage"] = 90
    data["context_menu_icon"] = True
    data["show_download_window"] = True
    data["show_terminal"] = False
    data["file_ext"] = dict.fromkeys(data["file_ext"], True)
    data["providers"] = dict.fromkeys(data["providers"], True)
    config_file = f"{__data__}\\config.json"
    with open(config_file, "r+", encoding="utf-8") as file:
        file.seek(0)
        json.dump(data, file, indent=4)
        file.truncate()


class UserParameters(NamedTuple):
    current_language: str
    hearing_impaired: bool | str
    pct: int
    show_dl_window: bool



import tkinter as tk

from subsearch.data import __icons__, __version__
from subsearch.gui import tk_data, tk_tools

TKWINDOW = tk_data.Window()
TKCOLOR = tk_data.Color()


def main():
    global root
    root = tk.Tk(className=f"Subsearch")
    root.configure(background=TKCOLOR.black)
    icon_path = f"{__icons__}\\16.ico"
    root.iconbitmap(icon_path)
    root.geometry(tk_tools.WindowPosition.set(root))
    root.resizable(False, False)
    tk_tools.CustomBorder(root).place(relx=0.5, rely=0.5, anchor="center")
    tk.Frame(root, height=40, bg=TKCOLOR.dark_grey).pack(anchor="center", expand=True)
    tk_tools.CustomTitleBar(root).place(x=TKWINDOW.width - 2, y=2, bordermode="inside", anchor="ne")

    return root
